@@ -1,4 +1,3 @@
-#include "config.h"
 #include <stdlib.h>
 #include <gegl.h>
 #include <glib/gprintf.h>
@@ -11,6 +10,7 @@ main (gint    argc,
       gchar **argv)
 {
   GeglNode   *gegl;   /* the gegl graph we're using as a node factor */
+  GeglBuffer *buffer;
   GeglNode   *display,
              *text,
              *layer,
@@ -31,6 +31,8 @@ main (gint    argc,
   gegl = gegl_node_new ();
 
 
+  buffer = gegl_buffer_open (argv[1]);
+
   blank      = gegl_node_new_child (gegl,
                                     "operation", "gegl:color",
                                     "value", gegl_color_new ("rgba(0.0,0.0,0.0,0.4)"),
@@ -57,11 +59,11 @@ main (gint    argc,
   text       = gegl_node_new_child (gegl,
                                    "operation", "gegl:text",
                                    "size", 20.0,
-                             /*      "color", gegl_color_new ("rgb(0.0,0.0,0.0)"),*/
+                                   "color", gegl_color_new ("rgb(1.0,1.0,1.0)"),
                                    NULL);
   display    = gegl_node_new_child (gegl,
-                                    "operation", "gegl:composite-buffer",
-                                    "path", argv[1],
+                                    "operation", "gegl:write-buffer",
+                                    "buffer", buffer,
                                     NULL);
 
   gegl_node_link_many (blank, crop, layer, shift, display, NULL);
