@@ -19,21 +19,20 @@
 #include "config.h"
 
 #include <glib-object.h>
+#include <glib/gstdio.h>
 
 #include "gegl.h"
 #include "gegl-buffer-types.h"
 #include "gegl-tile-storage.h"
 #include "gegl-tile.h"
-#include "gegl-tile-backend-file.h"
-#include "gegl-tile-backend-ram.h"
-#include "gegl-tile-backend-tiledir.h"
 #include "gegl-tile-handler-empty.h"
 #include "gegl-tile-handler-zoom.h"
 #include "gegl-tile-handler-cache.h"
 #include "gegl-tile-handler-log.h"
+#include "gegl-tile-handler-private.h"
 #include "gegl-types-internal.h"
-#include "gegl-utils.h"
 #include "gegl-config.h"
+#include "gegl-buffer-private.h"
 
 
 G_DEFINE_TYPE (GeglTileStorage, gegl_tile_storage, GEGL_TYPE_TILE_HANDLER_CHAIN)
@@ -48,41 +47,15 @@ enum
 
 guint gegl_tile_storage_signals[LAST_SIGNAL] = { 0 };
 
-static gboolean
-tile_storage_idle (gpointer data)
-{
-  GeglTileStorage *tile_storage = GEGL_TILE_STORAGE (data);
-
-  if (0 /* nothing to do*/)
-    {
-      tile_storage->idle_swapper = 0;
-      return FALSE;
-    }
-
-  return gegl_tile_source_idle (GEGL_TILE_SOURCE (tile_storage));
-}
-
-GeglTileBackend *gegl_buffer_backend (GObject *buffer);
-
 GeglTileStorage *
 gegl_tile_storage_new (GeglTileBackend *backend)
 {
   GeglTileStorage *tile_storage = g_object_new (GEGL_TYPE_TILE_STORAGE, NULL);
   GeglTileHandlerChain  *tile_handler_chain;
   GeglTileHandler       *handler;
+  GeglTileHandler       *cache = NULL;
   GeglTileHandler       *empty = NULL;
   GeglTileHandler       *zoom = NULL;
-  GeglTileHandlerCache  *cache = NULL;
-
-  tile_storage->seen_zoom = 0;
-  tile_storage->mutex = g_mutex_new ();
-  tile_storage->width = G_MAXINT;
-  tile_storage->height = G_MAXINT;
-
-  if (g_object_class_find_property (G_OBJECT_GET_CLASS (backend), "path"))
-    {
-      g_object_get (backend, "path", &tile_storage->path, NULL);
-    }
 
   tile_handler_chain = GEGL_TILE_HANDLER_CHAIN (tile_storage);
   handler  = GEGL_TILE_HANDLER (tile_storage);
@@ -93,69 +66,72 @@ gegl_tile_storage_new (GeglTileBackend *backend)
   tile_storage->format      = gegl_tile_backend_get_format (backend);
   tile_storage->tile_size   = gegl_tile_backend_get_tile_size (backend);
 
-  gegl_tile_handler_set_source (handler, (void*)backend);
+  gegl_tile_handler_set_source (handler, GEGL_TILE_SOURCE (backend));
 
-  { /* should perhaps be a.. method on gegl_tile_handler_chain_set_source
-       wrapping handler_set_source() and this*/
-    GeglTileHandlerChain *tile_handler_chain2 = GEGL_TILE_HANDLER_CHAIN (handler);
-    GSList         *iter   = (void *) tile_handler_chain2->chain;
-    while (iter && iter->next)
-      iter = iter->next;
-    if (iter)
-      {
-        gegl_tile_handler_set_source (GEGL_TILE_HANDLER (iter->data), handler->source);
-      }
-  }
+  cache = gegl_tile_handler_cache_new ();
+  empty = gegl_tile_handler_empty_new (backend);
+  zoom = gegl_tile_handler_zoom_new (backend);
 
-  g_object_unref (handler->source); /* eeek */
-  backend = GEGL_TILE_BACKEND (handler->source);
+  /* set storage and cache on ourself so chain_add() can use it */
+  _gegl_tile_handler_set_tile_storage (handler, tile_storage);
+  _gegl_tile_handler_set_cache (handler, (GeglTileHandlerCache *) cache);
 
-#if 0
-  if (g_getenv("GEGL_LOG_TILE_BACKEND")||
-      g_getenv("GEGL_TILE_LOG"))
-    gegl_tile_handler_chain_add (tile_handler_chain,
-                      g_object_new (GEGL_TYPE_TILE_HANDLER_LOG, NULL));
-#endif
-
-  cache = g_object_new (GEGL_TYPE_TILE_HANDLER_CACHE, NULL);
-  empty = gegl_tile_handler_empty_new (backend, cache);
-  zoom = gegl_tile_handler_zoom_new (backend, tile_storage, cache);
-
-  gegl_tile_handler_chain_add (tile_handler_chain, (void*)cache);
+  gegl_tile_handler_chain_add (tile_handler_chain, cache);
   gegl_tile_handler_chain_add (tile_handler_chain, zoom);
   gegl_tile_handler_chain_add (tile_handler_chain, empty);
 
-#if 0
-  if (g_getenv("GEGL_LOG_TILE_CACHE"))
-    gegl_tile_handler_chain_add (tile_handler_chain,
-                                 g_object_new (GEGL_TYPE_TILE_HANDLER_LOG, NULL));
-#endif
-  tile_storage->cache = cache;
+  g_object_unref (cache);
+  g_object_unref (zoom);
+  g_object_unref (empty);
+
+  tile_storage->cache = (GeglTileHandlerCache *) cache;
+  ((GeglTileHandlerCache *) cache)->tile_storage = tile_storage;
   gegl_tile_handler_chain_bind (tile_handler_chain);
 
-  ((GeglTileBackend *)gegl_buffer_backend ((void*)tile_storage))->priv->storage = (gpointer)
-                                             tile_storage;
-
-  tile_storage->idle_swapper = g_timeout_add_full (G_PRIORITY_LOW,
-                                              250,
-                                              tile_storage_idle,
-                                              tile_storage,
-                                              NULL);
+  backend->priv->storage = tile_storage;
 
   return tile_storage;
 }
 
+void
+gegl_tile_storage_add_handler (GeglTileStorage *tile_storage,
+                               GeglTileHandler *handler)
+{
+  GeglTileHandlerChain *chain = GEGL_TILE_HANDLER_CHAIN (tile_storage);
+
+  g_return_if_fail (GEGL_IS_TILE_HANDLER (handler));
+
+  gegl_tile_handler_chain_add (chain, handler);
+
+  /* FIXME: Move the handler to before the cache and other custom handlers */
+  chain->chain = g_slist_remove (chain->chain, handler);
+  chain->chain = g_slist_insert (chain->chain, handler, 2);
+
+  gegl_tile_handler_chain_bind (chain);
+}
+
+void
+gegl_tile_storage_remove_handler (GeglTileStorage *tile_storage,
+                                  GeglTileHandler *handler)
+{
+  GeglTileHandlerChain *chain = GEGL_TILE_HANDLER_CHAIN (tile_storage);
+
+  g_return_if_fail (GEGL_IS_TILE_HANDLER (handler));
+  g_return_if_fail (g_slist_find (chain->chain, handler));
+
+  chain->chain = g_slist_remove (chain->chain, handler);
+  gegl_tile_handler_set_source (handler, NULL);
+  g_object_unref (handler);
+
+  gegl_tile_handler_chain_bind (chain);
+}
+
 static void
 gegl_tile_storage_finalize (GObject *object)
 {
   GeglTileStorage *self = GEGL_TILE_STORAGE (object);
 
-  if (self->idle_swapper)
-    g_source_remove (self->idle_swapper);
-
-  if (self->path)
-    g_free (self->path);
-  g_mutex_free (self->mutex);
+  g_rec_mutex_clear (&self->mutex);
 
   (*G_OBJECT_CLASS (parent_class)->finalize)(object);
 }
@@ -180,6 +156,8 @@ gegl_tile_storage_class_init (GeglTileStorageClass *class)
 }
 
 static void
-gegl_tile_storage_init (GeglTileStorage *buffer)
+gegl_tile_storage_init (GeglTileStorage *tile_storage)
 {
+  tile_storage->seen_zoom = 0;
+  g_rec_mutex_init (&tile_storage->mutex);
 }
