@@ -25,37 +25,17 @@
 #include <glib.h>
 #include <glib-object.h>
 #include "gegl-paramspecs.h"
+#include "gegl-types.h"
+#include <babl/babl.h>
+#include "gegl-color.h"
+#include "gegl-curve.h"
+#include "gegl-path.h"
 
-
-/*
- * GEGL_TYPE_INT32
- */
+static void       gegl_param_double_class_init (GParamSpecClass *klass);
+static void       gegl_param_double_init       (GParamSpec      *pspec);
 
 GType
-gegl_int32_get_type (void)
-{
-  static GType type = 0;
-
-  if (!type)
-    {
-      const GTypeInfo info = { 0, };
-
-      type = g_type_register_static (G_TYPE_INT, "GeglInt32", &info, 0);
-    }
-
-  return type;
-}
-
-
-/*
- * GEGL_TYPE_PARAM_INT32
- */
-
-static void   gegl_param_int32_class_init (GParamSpecClass *klass);
-static void   gegl_param_int32_init (GParamSpec *pspec);
-
-GType
-gegl_param_int32_get_type (void)
+gegl_param_double_get_type (void)
 {
   static GType type = 0;
 
@@ -64,238 +44,172 @@ gegl_param_int32_get_type (void)
       const GTypeInfo info =
       {
         sizeof (GParamSpecClass),
-        NULL,                                         NULL,
-        (GClassInitFunc) gegl_param_int32_class_init,
-        NULL,                                         NULL,
-        sizeof (GeglParamSpecInt32),
+        NULL, NULL,
+        (GClassInitFunc) gegl_param_double_class_init,
+        NULL, NULL,
+        sizeof (GeglParamSpecDouble),
         0,
-        (GInstanceInitFunc) gegl_param_int32_init
+        (GInstanceInitFunc) gegl_param_double_init
       };
+      type = g_type_register_static (G_TYPE_PARAM_DOUBLE,
+                                     "GeglParamDouble", &info, 0);
+    }
+  return type;
+}
 
+static void
+gegl_param_double_class_init (GParamSpecClass *klass)
+{
+  klass->value_type     = G_TYPE_DOUBLE;
+}
+
+static void
+gegl_param_double_init (GParamSpec *pspec)
+{
+  GParamSpecDouble    *dpspec  = G_PARAM_SPEC_DOUBLE (pspec);
+  GeglParamSpecDouble *gdpspec = GEGL_PARAM_SPEC_DOUBLE (pspec);
+  gdpspec->ui_minimum = dpspec->minimum;
+  gdpspec->ui_maximum = dpspec->maximum;
+  gdpspec->ui_gamma   = 1.0;
+}
+
+GParamSpec *
+gegl_param_spec_double (const gchar *name,
+                        const gchar *nick,
+                        const gchar *blurb,
+                        gdouble      minimum,
+                        gdouble      maximum,
+                        gdouble      default_value,
+                        gdouble      ui_minimum,
+                        gdouble      ui_maximum,
+                        gdouble      ui_gamma,
+                        GParamFlags  flags)
+{
+  GeglParamSpecDouble *pspec;
+  GParamSpecDouble *dspec;
+
+  pspec = g_param_spec_internal (GEGL_TYPE_PARAM_DOUBLE,
+                                 name, nick, blurb, flags);
+  dspec = G_PARAM_SPEC_DOUBLE (pspec);
+
+  dspec->minimum = minimum;
+  dspec->maximum = maximum;
+  dspec->default_value = default_value;
+  pspec->ui_minimum = ui_minimum;
+  pspec->ui_maximum = ui_maximum;
+  pspec->ui_gamma   = ui_gamma;
+  gegl_param_spec_double_set_steps (pspec, 0.1, 1.0);
+  gegl_param_spec_double_set_digits (pspec, 2);
+
+  return G_PARAM_SPEC (pspec);
+}
+
+void
+gegl_param_spec_double_set_steps (GeglParamSpecDouble *pspec,
+                                  gdouble              step_small,
+                                  gdouble              step_big)
+{
+  g_return_if_fail (GEGL_IS_PARAM_SPEC_DOUBLE (pspec));
+
+  pspec->ui_step_small = step_small;
+  pspec->ui_step_big = step_big;
+}
+
+void gegl_param_spec_double_set_digits (GeglParamSpecDouble *pspec,
+                                        gint                 digits)
+{
+  g_return_if_fail (GEGL_IS_PARAM_SPEC_DOUBLE (pspec));
+
+  pspec->ui_digits = digits;
+}
+
+gdouble gegl_param_spec_double_get_step_size (GeglParamSpecDouble *pspec);
+gdouble gegl_param_spec_double_get_page_size (GeglParamSpecDouble *pspec);
+
+static void       gegl_param_int_class_init (GParamSpecClass *klass);
+static void       gegl_param_int_init       (GParamSpec      *pspec);
+
+GType
+gegl_param_int_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+    {
+      const GTypeInfo info =
+      {
+        sizeof (GParamSpecClass),
+        NULL, NULL,
+        (GClassInitFunc) gegl_param_int_class_init,
+        NULL, NULL,
+        sizeof (GeglParamSpecInt),
+        0,
+        (GInstanceInitFunc) gegl_param_int_init
+      };
       type = g_type_register_static (G_TYPE_PARAM_INT,
-                                     "GeglParamInt32", &info, 0);
+                                     "GeglParamInt", &info, 0);
     }
-
   return type;
 }
 
 static void
-gegl_param_int32_class_init (GParamSpecClass *klass)
+gegl_param_int_class_init (GParamSpecClass *klass)
 {
-  klass->value_type = GEGL_TYPE_INT32;
+  klass->value_type     = G_TYPE_INT;
 }
 
 static void
-gegl_param_int32_init (GParamSpec *pspec)
+gegl_param_int_init (GParamSpec *pspec)
 {
+  GParamSpecInt    *dpspec  = G_PARAM_SPEC_INT (pspec);
+  GeglParamSpecInt *gdpspec = GEGL_PARAM_SPEC_INT (pspec);
+  gdpspec->ui_minimum = dpspec->minimum;
+  gdpspec->ui_maximum = dpspec->maximum;
+  gdpspec->ui_gamma = 1.0;
 }
 
 GParamSpec *
-gegl_param_spec_int32 (const gchar *name,
-                       const gchar *nick,
-                       const gchar *blurb,
-                       gint         minimum,
-                       gint         maximum,
-                       gint         default_value,
-                       GParamFlags  flags)
+gegl_param_spec_int (const gchar *name,
+                     const gchar *nick,
+                     const gchar *blurb,
+                     gint         minimum,
+                     gint         maximum,
+                     gint         default_value,
+                     gint         ui_minimum,
+                     gint         ui_maximum,
+                     gdouble      ui_gamma,
+                     GParamFlags  flags)
 {
+  GeglParamSpecInt *pspec;
   GParamSpecInt *ispec;
 
-  g_return_val_if_fail (minimum >= G_MININT32, NULL);
-  g_return_val_if_fail (maximum <= G_MAXINT32, NULL);
-  g_return_val_if_fail (default_value >= minimum &&
-                        default_value <= maximum, NULL);
 
-  ispec = g_param_spec_internal (GEGL_TYPE_PARAM_INT32,
+  pspec = g_param_spec_internal (GEGL_TYPE_PARAM_INT,
                                  name, nick, blurb, flags);
+  ispec = G_PARAM_SPEC_INT (pspec);
 
-  ispec->minimum       = minimum;
-  ispec->maximum       = maximum;
+  ispec->minimum = minimum;
+  ispec->maximum = maximum;
   ispec->default_value = default_value;
+  pspec->ui_minimum = ui_minimum;
+  pspec->ui_maximum = ui_maximum;
+  pspec->ui_gamma = ui_gamma;
 
-  return G_PARAM_SPEC (ispec);
+  gegl_param_spec_int_set_steps (pspec, 1, 5);
+  return G_PARAM_SPEC (pspec);
 }
 
-
-/*
- * GEGL_TYPE_INT16
- */
-
-GType
-gegl_int16_get_type (void)
+void
+gegl_param_spec_int_set_steps (GeglParamSpecInt *pspec,
+                               gint              step_small,
+                               gint              step_big)
 {
-  static GType type = 0;
+  g_return_if_fail (GEGL_IS_PARAM_SPEC_INT (pspec));
 
-  if (!type)
-    {
-      const GTypeInfo info = { 0, };
-
-      type = g_type_register_static (G_TYPE_INT, "GeglInt16", &info, 0);
-    }
-
-  return type;
+  pspec->ui_step_small = step_small;
+  pspec->ui_step_big = step_big;
 }
 
-
-/*
- * GEGL_TYPE_PARAM_INT16
- */
-
-static void   gegl_param_int16_class_init (GParamSpecClass *klass);
-static void   gegl_param_int16_init (GParamSpec *pspec);
-
-GType
-gegl_param_int16_get_type (void)
-{
-  static GType type = 0;
-
-  if (!type)
-    {
-      const GTypeInfo info =
-      {
-        sizeof (GParamSpecClass),
-        NULL,                                         NULL,
-        (GClassInitFunc) gegl_param_int16_class_init,
-        NULL,                                         NULL,
-        sizeof (GeglParamSpecInt16),
-        0,
-        (GInstanceInitFunc) gegl_param_int16_init
-      };
-
-      type = g_type_register_static (G_TYPE_PARAM_INT,
-                                     "GeglParamInt16", &info, 0);
-    }
-
-  return type;
-}
-
-static void
-gegl_param_int16_class_init (GParamSpecClass *klass)
-{
-  klass->value_type = GEGL_TYPE_INT16;
-}
-
-static void
-gegl_param_int16_init (GParamSpec *pspec)
-{
-}
-
-GParamSpec *
-gegl_param_spec_int16 (const gchar *name,
-                       const gchar *nick,
-                       const gchar *blurb,
-                       gint         minimum,
-                       gint         maximum,
-                       gint         default_value,
-                       GParamFlags  flags)
-{
-  GParamSpecInt *ispec;
-
-  g_return_val_if_fail (minimum >= G_MININT16, NULL);
-  g_return_val_if_fail (maximum <= G_MAXINT16, NULL);
-  g_return_val_if_fail (default_value >= minimum &&
-                        default_value <= maximum, NULL);
-
-  ispec = g_param_spec_internal (GEGL_TYPE_PARAM_INT16,
-                                 name, nick, blurb, flags);
-
-  ispec->minimum       = minimum;
-  ispec->maximum       = maximum;
-  ispec->default_value = default_value;
-
-  return G_PARAM_SPEC (ispec);
-}
-
-
-/*
- * GEGL_TYPE_INT8
- */
-
-GType
-gegl_int8_get_type (void)
-{
-  static GType type = 0;
-
-  if (!type)
-    {
-      const GTypeInfo info = { 0, };
-
-      type = g_type_register_static (G_TYPE_UINT, "GeglInt8", &info, 0);
-    }
-
-  return type;
-}
-
-
-/*
- * GEGL_TYPE_PARAM_INT8
- */
-
-static void   gegl_param_int8_class_init (GParamSpecClass *klass);
-static void   gegl_param_int8_init (GParamSpec *pspec);
-
-GType
-gegl_param_int8_get_type (void)
-{
-  static GType type = 0;
-
-  if (!type)
-    {
-      const GTypeInfo info =
-      {
-        sizeof (GParamSpecClass),
-        NULL,                                        NULL,
-        (GClassInitFunc) gegl_param_int8_class_init,
-        NULL,                                        NULL,
-        sizeof (GeglParamSpecInt8),
-        0,
-        (GInstanceInitFunc) gegl_param_int8_init
-      };
-
-      type = g_type_register_static (G_TYPE_PARAM_UINT,
-                                     "GeglParamInt8", &info, 0);
-    }
-
-  return type;
-}
-
-static void
-gegl_param_int8_class_init (GParamSpecClass *klass)
-{
-  klass->value_type = GEGL_TYPE_INT8;
-}
-
-static void
-gegl_param_int8_init (GParamSpec *pspec)
-{
-}
-
-GParamSpec *
-gegl_param_spec_int8 (const gchar *name,
-                      const gchar *nick,
-                      const gchar *blurb,
-                      guint        minimum,
-                      guint        maximum,
-                      guint        default_value,
-                      GParamFlags  flags)
-{
-  GParamSpecInt *ispec;
-
-  g_return_val_if_fail (maximum <= G_MAXUINT8, NULL);
-  g_return_val_if_fail (default_value >= minimum &&
-                        default_value <= maximum, NULL);
-
-  ispec = g_param_spec_internal (GEGL_TYPE_PARAM_INT8,
-                                 name, nick, blurb, flags);
-
-  ispec->minimum       = minimum;
-  ispec->maximum       = maximum;
-  ispec->default_value = default_value;
-
-  return G_PARAM_SPEC (ispec);
-}
-
-
 /*
  * GEGL_TYPE_PARAM_STRING
  */
@@ -402,6 +316,7 @@ gegl_param_spec_string (const gchar *name,
   return G_PARAM_SPEC (sspec);
 }
 
+
 /*
  * GEGL_TYPE_PARAM_FILE_PATH
  */
@@ -744,3 +659,232 @@ gegl_param_spec_enum_exclude_value (GeglParamSpecEnum *espec,
   espec->excluded_values = g_slist_prepend (espec->excluded_values,
                                             GINT_TO_POINTER (value));
 }
+
+/*
+ * GEGL_TYPE_PARAM_SEED
+ */
+
+static void       gegl_param_seed_class_init (GParamSpecClass *klass);
+static void       gegl_param_seed_init       (GParamSpec      *pspec);
+
+GType
+gegl_param_seed_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+    {
+      const GTypeInfo info =
+      {
+        sizeof (GParamSpecClass),
+        NULL, NULL,
+        (GClassInitFunc) gegl_param_seed_class_init,
+        NULL, NULL,
+        sizeof (GeglParamSpecSeed),
+        0,
+        (GInstanceInitFunc) gegl_param_seed_init
+      };
+      type = g_type_register_static (G_TYPE_PARAM_INT,
+                                     "GeglParamSeed", &info, 0);
+    }
+  return type;
+}
+
+static void
+gegl_param_seed_class_init (GParamSpecClass *klass)
+{
+  klass->value_type = G_TYPE_INT;
+}
+
+static void
+gegl_param_seed_init (GParamSpec *pspec)
+{
+  GeglParamSpecSeed *gdpspec = GEGL_PARAM_SPEC_SEED (pspec);
+
+  gdpspec->ui_minimum = 0;
+  gdpspec->ui_maximum = G_MAXINT;
+}
+
+GParamSpec *
+gegl_param_spec_seed (const gchar *name,
+                      const gchar *nick,
+                      const gchar *blurb,
+                      GParamFlags  flags)
+{
+  GeglParamSpecSeed *pspec;
+  GParamSpecInt     *ispec;
+
+  pspec = g_param_spec_internal (GEGL_TYPE_PARAM_SEED,
+                                 name, nick, blurb, flags);
+  ispec = G_PARAM_SPEC_INT (pspec);
+
+  ispec->minimum = 0;
+  ispec->maximum = G_MAXINT;
+  ispec->default_value = 0;
+  pspec->ui_minimum = 0;
+  pspec->ui_maximum = G_MAXINT;
+
+  return G_PARAM_SPEC (pspec);
+}
+
+/*
+ * GEGL_TYPE_PARAM_FORMAT
+ */
+
+static void       gegl_param_format_class_init (GParamSpecClass *klass);
+static void       gegl_param_format_init       (GParamSpec      *pspec);
+
+GType
+gegl_param_format_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+    {
+      const GTypeInfo info =
+      {
+        sizeof (GParamSpecClass),
+        NULL, NULL,
+        (GClassInitFunc) gegl_param_format_class_init,
+        NULL, NULL,
+        sizeof (GeglParamSpecFormat),
+        0,
+        (GInstanceInitFunc) gegl_param_format_init
+      };
+      type = g_type_register_static (G_TYPE_PARAM_POINTER,
+                                     "GeglParamFormat", &info, 0);
+    }
+  return type;
+}
+
+static void
+gegl_param_format_class_init (GParamSpecClass *klass)
+{
+  klass->value_type = G_TYPE_POINTER;
+}
+
+static void
+gegl_param_format_init (GParamSpec *pspec)
+{
+}
+
+GParamSpec *
+gegl_param_spec_format (const gchar *name,
+                        const gchar *nick,
+                        const gchar *blurb,
+                        GParamFlags  flags)
+{
+  GeglParamSpecFormat *pspec;
+
+  pspec = g_param_spec_internal (GEGL_TYPE_PARAM_FORMAT,
+                                 name, nick, blurb, flags);
+
+  return G_PARAM_SPEC (pspec);
+}
+
+/*
+ * GEGL_TYPE_PARAM_URI
+ */
+
+static void       gegl_param_uri_class_init (GParamSpecClass *klass);
+static void       gegl_param_uri_init       (GParamSpec *pspec);
+static gboolean   gegl_param_uri_validate   (GParamSpec *pspec,
+                                                   GValue     *value);
+
+GType
+gegl_param_uri_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+    {
+      const GTypeInfo info =
+      {
+        sizeof (GParamSpecClass),
+        NULL,                                        NULL,
+        (GClassInitFunc) gegl_param_uri_class_init,
+        NULL,                                        NULL,
+        sizeof (GeglParamSpecString),
+        0,
+        (GInstanceInitFunc) gegl_param_uri_init
+      };
+
+      type = g_type_register_static (G_TYPE_PARAM_STRING,
+                                     "GeglParamUri", &info, 0);
+    }
+
+  return type;
+}
+
+static void
+gegl_param_uri_class_init (GParamSpecClass *klass)
+{
+  klass->value_type     = G_TYPE_STRING;
+  klass->value_validate = gegl_param_uri_validate;
+}
+
+static void
+gegl_param_uri_init (GParamSpec *pspec)
+{
+  GeglParamSpecUri *sspec = GEGL_PARAM_SPEC_URI (pspec);
+
+  sspec->no_validate = FALSE;
+  sspec->null_ok     = FALSE;
+}
+
+static gboolean
+gegl_param_uri_validate (GParamSpec *pspec,
+                               GValue     *value)
+{
+  GeglParamSpecUri *sspec = GEGL_PARAM_SPEC_URI (pspec);
+  gchar                 *path  = value->data[0].v_pointer;
+
+  if (path)
+    {
+      gchar *s;
+
+      if (!sspec->no_validate &&
+          !g_utf8_validate (path, -1, (const gchar **) &s))
+        {
+          for (; *s; s++)
+            if (*s < ' ')
+              *s = '?';
+
+          return TRUE;
+        }
+    }
+  else if (!sspec->null_ok)
+    {
+      value->data[0].v_pointer = g_strdup ("");
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+GParamSpec *
+gegl_param_spec_uri (const gchar *name,
+                           const gchar *nick,
+                           const gchar *blurb,
+                           gboolean     no_validate,
+                           gboolean     null_ok,
+                           const gchar *default_value,
+                           GParamFlags  flags)
+{
+  GeglParamSpecUri *sspec;
+
+  sspec = g_param_spec_internal (GEGL_TYPE_PARAM_URI,
+                                 name, nick, blurb, flags);
+
+  if (sspec)
+    {
+      g_free (G_PARAM_SPEC_STRING (sspec)->default_value);
+      G_PARAM_SPEC_STRING (sspec)->default_value = g_strdup (default_value);
+
+      sspec->no_validate = no_validate ? TRUE : FALSE;
+      sspec->null_ok     = null_ok     ? TRUE : FALSE;
+    }
+
+  return G_PARAM_SPEC (sspec);
+}
+
