@@ -21,16 +21,19 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_int_ui (iterations, _("Strength"), 0, 32, 4, 0, 8, 1, _("How many iteratarions to run the algorithm with"))
+property_int  (iterations, _("Strength"), 4)
+  description (_("Controls the number of iterations; lower values give less plastic results"))
+  value_range (1, 32)
+  ui_range    (1, 8)
 
 #else
 
-#define GEGL_CHANT_TYPE_AREA_FILTER
-#define GEGL_CHANT_C_FILE       "noise-reduction.c"
+#define GEGL_OP_AREA_FILTER
+#define GEGL_OP_C_SOURCE noise-reduction.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <math.h>
 
 /* The core noise_reduction function, which is implemented as
@@ -142,7 +145,7 @@ noise_reduction (float *src_buf,     /* source buffer, one pixel to the left
 static void prepare (GeglOperation *operation)
 {
   GeglOperationAreaFilter *area = GEGL_OPERATION_AREA_FILTER (operation);
-  GeglChantO              *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties              *o = GEGL_PROPERTIES (operation);
 
   area->left = area->right = area->top = area->bottom = o->iterations;
   gegl_operation_set_format (operation, "input",  babl_format ("R'G'B'A float"));
@@ -171,7 +174,8 @@ cl_noise_reduction (cl_mem                in_tex,
   cl_int n_src_stride  = roi->width + iterations * 2;
   cl_int cl_err = 0;
 
-  cl_mem temp_tex, tmptex;
+  cl_mem temp_tex = NULL;
+  cl_mem tmptex   = NULL;
 
   gint stride = 16; /*R'G'B'A float*/
 
@@ -239,12 +243,18 @@ cl_noise_reduction (cl_mem                in_tex,
   cl_err = gegl_clFinish(gegl_cl_get_command_queue());
   CL_CHECK;
 
-  if (tmptex) cl_err = gegl_clReleaseMemObject (tmptex);
-  CL_CHECK;
+  if (tmptex)
+    {
+      cl_err = gegl_clReleaseMemObject (tmptex);
+      CL_CHECK_ONLY (cl_err);
+    }
 
   return FALSE;
 
 error:
+  if (tmptex)
+    gegl_clReleaseMemObject (tmptex);
+
   return TRUE;
 }
 
@@ -256,38 +266,53 @@ cl_process (GeglOperation       *operation,
 {
   const Babl *in_format  = gegl_operation_get_format (operation, "input");
   const Babl *out_format = gegl_operation_get_format (operation, "output");
-  gint err;
-  gint j;
+  gint err = 0;
 
   GeglOperationAreaFilter *op_area = GEGL_OPERATION_AREA_FILTER (operation);
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
 
-  GeglBufferClIterator *i = gegl_buffer_cl_iterator_new (output,   result, out_format, GEGL_CL_BUFFER_WRITE);
-  gint read = gegl_buffer_cl_iterator_add_2 (i, input, result, in_format,  GEGL_CL_BUFFER_READ,
-                                             op_area->left, op_area->right, op_area->top, op_area->bottom, GEGL_ABYSS_NONE);
-  gint aux  = gegl_buffer_cl_iterator_add_2 (i, NULL, result, in_format,  GEGL_CL_BUFFER_AUX,
-                                             op_area->left, op_area->right, op_area->top, op_area->bottom, GEGL_ABYSS_NONE);
+  GeglBufferClIterator *i = gegl_buffer_cl_iterator_new (output,
+                                                         result,
+                                                         out_format,
+                                                         GEGL_CL_BUFFER_WRITE);
 
-  while (gegl_buffer_cl_iterator_next (i, &err))
-  {
-    if (err) return FALSE;
-    for (j=0; j < i->n; j++)
-      {
-        err = cl_noise_reduction(i->tex[read][j],
-                                 i->tex[aux][j],
-                                 i->tex[0][j],
-                                 i->size[0][j],
-                                 &i->roi[read][j],
-                                 &i->roi[0][j],
-                                 o->iterations);
-        if (err)
+  gint read = gegl_buffer_cl_iterator_add_2 (i,
+                                             input,
+                                             result,
+                                             in_format,
+                                             GEGL_CL_BUFFER_READ,
+                                             op_area->left,
+                                             op_area->right,
+                                             op_area->top,
+                                             op_area->bottom,
+                                             GEGL_ABYSS_NONE);
+
+  gint aux = gegl_buffer_cl_iterator_add_aux (i,
+                                              result,
+                                              in_format,
+                                              op_area->left,
+                                              op_area->right,
+                                              op_area->top,
+                                              op_area->bottom);
+
+  while (gegl_buffer_cl_iterator_next (i, &err) && !err)
+    {
+      err = cl_noise_reduction (i->tex[read],
+                                i->tex[aux],
+                                i->tex[0],
+                                i->size[0],
+                                &i->roi[read],
+                                &i->roi[0],
+                                o->iterations);
+
+      if (err)
         {
-          g_warning("[OpenCL] Error in gegl:noise-reduction");
-          return FALSE;
+          gegl_buffer_cl_iterator_stop (i);
+          break;
         }
-      }
-  }
-  return TRUE;
+    }
+
+  return !err;
 }
 
 #define INPLACE 1
@@ -299,7 +324,7 @@ process (GeglOperation       *operation,
          const GeglRectangle *result,
          gint                 level)
 {
-  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties   *o = GEGL_PROPERTIES (operation);
 
   int iteration;
   int stride;
@@ -309,7 +334,7 @@ process (GeglOperation       *operation,
 #endif
   GeglRectangle rect;
 
-  if (gegl_cl_is_accelerated ())
+  if (gegl_operation_use_opencl (operation))
     if(cl_process(operation, input, output, result))
       return TRUE;
 
@@ -382,7 +407,7 @@ get_bounding_box (GeglOperation *operation)
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationFilterClass *filter_class;
@@ -397,9 +422,10 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_bounding_box = get_bounding_box;
 
   gegl_operation_class_set_keys (operation_class,
+    "title",       _("Noise Reduction"),
     "name"       , "gegl:noise-reduction",
-    "categories" , "enhance",
-    "description", "Anisotropic like smoothing operation",
+    "categories" , "enhance:noise-reduction",
+    "description", _("Anisotropic smoothing operation"),
     NULL);
 }
 
