@@ -19,21 +19,30 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_double_ui (std_dev, _("Std. Dev."), 0.0, 500.0, 1.0, 0.0, 200.0, 1.5,
-                  _("Standard deviation (spatial scale factor)"))
-gegl_chant_double_ui (scale,  _("Scale"), 0.0, 100.0, 1.0, 0.0, 100.0, 1.5,
-                  _("Scale, strength of effect"))
+property_double (std_dev, _("Standard Deviation"), 0.55)
+    description(_("Standard deviation (spatial scale factor)"))
+    value_range (0.2, 300)
+    ui_range    (0.2, 40.0)
+    ui_gamma    (3.0)
+    ui_meta     ("unit", "pixel-distance")
+
+property_double (scale, _("Scale"), 4.0)
+    description(_("Scale, strength of effect"))
+    value_range (0.0, 300.0)
+    ui_range    (0.0, 10.0)
+    ui_gamma    (3.0)
 
 #else
 
-#define GEGL_CHANT_TYPE_META
-#define GEGL_CHANT_C_FILE       "unsharp-mask.c"
+#define GEGL_OP_META
+#define GEGL_OP_C_SOURCE unsharp-mask.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
-static void attach (GeglOperation *operation)
+static void
+attach (GeglOperation *operation)
 {
   GeglNode *gegl, *input, *output, *add, *multiply, *subtract, *blur;
 
@@ -56,23 +65,25 @@ static void attach (GeglOperation *operation)
   gegl_operation_meta_redirect (operation, "scale", multiply, "value");
   gegl_operation_meta_redirect (operation, "std-dev", blur, "std-dev-x");
   gegl_operation_meta_redirect (operation, "std-dev", blur, "std-dev-y");
+
+  gegl_operation_meta_watch_nodes (operation, add, multiply, subtract, blur, NULL);
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass *operation_class;
 
   operation_class = GEGL_OPERATION_CLASS (klass);
+
   operation_class->attach = attach;
 
   gegl_operation_class_set_keys (operation_class,
-  "name"       , "gegl:unsharp-mask",
-  "categories" , "meta:enhance",
-  "description",
-        _("Performs an unsharp mask on the input buffer (sharpens an image by "
-          "adding false mach-bands around edges)"),
-        NULL);
+    "name",        "gegl:unsharp-mask",
+    "title",       _("Unsharp Mask"),
+    "categories",  "enhance:sharpen",
+    "description", _("Sharpen image, by adding difference to blurred image, a technique for sharpening originally used in darkrooms."),
+    NULL);
 }
 
 #endif
