@@ -16,6 +16,12 @@
  * Copyright 2007 Mukund Sivaraman <muks@mukund.org>
  */
 
+/* XXX
+ * This plug-in isn't really useful apart for compatibility with GIMP
+ * scripts, operating with HSV as a color model for filters isn't really
+ * useful.
+ */
+
 /*
  * The plug-in only does v = 1.0 - v; for each pixel in the image, or
  * each entry in the colormap depending upon the type of image, where 'v'
@@ -34,24 +40,31 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
    /* no properties */
 
 #else
 
-#define GEGL_CHANT_TYPE_POINT_FILTER
-#define GEGL_CHANT_C_FILE       "value-invert.c"
+#define GEGL_OP_POINT_FILTER
+#define GEGL_OP_C_SOURCE value-invert.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
+
+static void
+prepare (GeglOperation *operation)
+{
+  gegl_operation_set_format (operation, "input", babl_format ("R'G'B'A float"));
+  gegl_operation_set_format (operation, "output", babl_format ("R'G'B'A float"));
+}
 
 static gboolean
 process (GeglOperation       *op,
          void                *in_buf,
          void                *out_buf,
          glong                samples,
-         const GeglRectangle *roi)
+         const GeglRectangle *roi,
+         gint                 level)
 {
   glong   j;
   gfloat *src  = in_buf;
@@ -115,9 +128,10 @@ process (GeglOperation       *op,
   return TRUE;
 }
 
+#include "opencl/value-invert.cl.h"
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass            *operation_class;
   GeglOperationPointFilterClass *point_filter_class;
@@ -126,12 +140,17 @@ gegl_chant_class_init (GeglChantClass *klass)
   point_filter_class = GEGL_OPERATION_POINT_FILTER_CLASS (klass);
 
   point_filter_class->process = process;
+  operation_class->prepare = prepare;
 
-  operation_class->name        = "gegl:value-invert";
-  operation_class->categories  = "color";
-  operation_class->description =
+  gegl_operation_class_set_keys (operation_class,
+    "name",        "gegl:value-invert",
+    "title",       _("Value Invert"),
+    "categories" , "color",
+    "description",
         _("Inverts just the value component, the result is the corresponding "
-          "`inverted' image.");
+          "`inverted' image."),
+    "cl-source"  , value_invert_cl_source,
+    NULL);
 }
 
 #endif
