@@ -20,37 +20,59 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_double (x,      _("X"),      -G_MAXFLOAT, G_MAXFLOAT,  0.0, _("X"))
-gegl_chant_double (y,      _("Y"),      -G_MAXFLOAT, G_MAXFLOAT,  0.0, _("Y"))
-gegl_chant_double (width,  _("Width"),  -G_MAXFLOAT, G_MAXFLOAT, 10.0, _("Width"))
-gegl_chant_double (height, _("Height"), -G_MAXFLOAT, G_MAXFLOAT, 10.0, _("Height"))
+property_double (x,      _("X"),      0.0)
+  ui_range      (0.0, 1024.0)
+  ui_meta       ("unit", "pixel-coordinate")
+  ui_meta       ("axis", "x")
+
+property_double (y,      _("Y"),      0.0)
+  ui_range      (0.0, 1024.0)
+  ui_meta       ("unit", "pixel-coordinate")
+  ui_meta       ("axis", "y")
+
+property_double (width,  _("Width"),  10.0 )
+  ui_range      (0.0, 1024.0)
+  ui_meta       ("unit", "pixel-distance")
+  ui_meta       ("axis", "x")
+
+property_double (height, _("Height"), 10.0 )
+  ui_range      (0.0, 1024.0)
+  ui_meta       ("unit", "pixel-distance")
+  ui_meta       ("axis", "y")
 
 #else
 
-#define GEGL_CHANT_TYPE_FILTER
-#define GEGL_CHANT_C_FILE       "crop.c"
+#define GEGL_OP_FILTER
+#define GEGL_OP_C_SOURCE crop.c
 
-#include "gegl-chant.h"
-#include "graph/gegl-node.h"
-#include "buffer/gegl-buffer-cl-cache.h"
+#include "gegl-op.h"
 #include <math.h>
 
+static void
+gegl_crop_prepare (GeglOperation *operation)
+{
+  const Babl *format = gegl_operation_get_source_format (operation, "input");
+
+  gegl_operation_set_format (operation, "input", format);
+  gegl_operation_set_format (operation, "output", format);
+}
+
 static GeglNode *
 gegl_crop_detect (GeglOperation *operation,
                   gint           x,
                   gint           y)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   GeglNode   *input_node;
 
   input_node = gegl_operation_get_source_node (operation, "input");
 
   if (input_node)
-    return gegl_operation_detect (input_node->operation,
-                                  x - floor (o->x),
-                                  y - floor (o->y));
+    return gegl_node_detect (input_node,
+                             x - floor (o->x),
+                             y - floor (o->y));
 
   return operation->node;
 }
@@ -59,15 +81,15 @@ gegl_crop_detect (GeglOperation *operation,
 static GeglRectangle
 gegl_crop_get_bounding_box (GeglOperation *operation)
 {
-  GeglChantO    *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties      *o = GEGL_PROPERTIES (operation);
   GeglRectangle *in_rect = gegl_operation_source_get_bounding_box (operation, "input");
   GeglRectangle  result  = { 0, 0, 0, 0 };
 
   if (!in_rect)
     return result;
 
-  result.x = o->x;
-  result.y = o->y;
+  result.x      = o->x;
+  result.y      = o->y;
   result.width  = o->width;
   result.height = o->height;
 
@@ -79,7 +101,7 @@ gegl_crop_get_invalidated_by_change (GeglOperation       *operation,
                                      const gchar         *input_pad,
                                      const GeglRectangle *input_region)
 {
-  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties   *o = GEGL_PROPERTIES (operation);
   GeglRectangle result;
 
   result.x = o->x;
@@ -97,12 +119,12 @@ gegl_crop_get_required_for_output (GeglOperation       *operation,
                                    const gchar         *input_pad,
                                    const GeglRectangle *roi)
 {
-  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties   *o = GEGL_PROPERTIES (operation);
   GeglRectangle result;
 
-  result.x = o->x;
-  result.y = o->y;
-  result.width = o->width;
+  result.x      = o->x;
+  result.y      = o->y;
+  result.width  = o->width;
   result.height = o->height;
 
   gegl_rectangle_intersect (&result, &result, roi);
@@ -116,32 +138,24 @@ gegl_crop_process (GeglOperation        *operation,
                    const GeglRectangle  *result,
                    gint                  level)
 {
-  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
-  GeglBuffer   *input;
-  gboolean      success = FALSE;
-  GeglRectangle extent;
+  GeglProperties *o = GEGL_PROPERTIES (operation);
+  GeglBuffer     *input;
+  gboolean        success = FALSE;
+  GeglRectangle   extent;
 
-  extent.x = o->x;
-  extent.y = o->y;
-  extent.width = o->width;
+  extent.x      = o->x;
+  extent.y      = o->y;
+  extent.width  = o->width;
   extent.height = o->height;
 
-  if (strcmp (output_prop, "output"))
+  input = gegl_operation_context_get_source (context, "input");
+
+  if (input)
     {
-      g_warning ("requested processing of %s pad on a crop", output_prop);
-      return FALSE;
-    }
+      GeglBuffer *output = gegl_buffer_create_sub_buffer (input, &extent);
 
-  input  = gegl_operation_context_get_source (context, "input");
-
-  if (input != NULL)
-    {
-      GeglBuffer *output;
-
-      output = gegl_buffer_create_sub_buffer (input, &extent);
-
-      if (gegl_object_get_has_forked (input))
-        gegl_object_set_has_forked (output);
+      if (gegl_object_get_has_forked (G_OBJECT (input)))
+        gegl_object_set_has_forked (G_OBJECT (output));
 
       gegl_operation_context_take_object (context, "output", G_OBJECT (output));
 
@@ -150,24 +164,38 @@ gegl_crop_process (GeglOperation        *operation,
     }
   else
     {
-      if (!g_object_get_data (G_OBJECT (operation->node), "graph"))
-        g_warning ("%s got %s",
-                   gegl_node_get_debug_name (operation->node),
-                   input==NULL?"input==NULL":"");
+      g_warning ("%s got NULL input pad", gegl_node_get_operation (operation->node));
     }
 
   return success;
 }
 
-
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass *operation_class;
+  gchar              *composition = "<?xml version='1.0' encoding='UTF-8'?>"
+    "<gegl>"
+    "<node operation='gegl:crop'>"
+    "  <params>"
+    "    <param name='x'>50</param>"
+    "    <param name='y'>80</param>"
+    "    <param name='width'>70</param>"
+    "    <param name='height'>60</param>"
+    "  </params>"
+    "</node>"
+    "<node operation='gegl:load'>"
+    "  <params>"
+    "    <param name='path'>standard-input.png</param>"
+    "  </params>"
+    "</node>"
+    "</gegl>";
 
   operation_class = GEGL_OPERATION_CLASS (klass);
 
+  operation_class->threaded                  = FALSE;
   operation_class->process                   = gegl_crop_process;
+  operation_class->prepare                   = gegl_crop_prepare;
   operation_class->get_bounding_box          = gegl_crop_get_bounding_box;
   operation_class->detect                    = gegl_crop_detect;
   operation_class->get_invalidated_by_change = gegl_crop_get_invalidated_by_change;
@@ -176,7 +204,9 @@ gegl_chant_class_init (GeglChantClass *klass)
   gegl_operation_class_set_keys (operation_class,
       "name",        "gegl:crop",
       "categories",  "core",
+      "title",       _("Crop"),
       "description", _("Crop a buffer"),
+      "reference-composition", composition,
       NULL);
 
   operation_class->no_cache = TRUE;
