@@ -25,18 +25,23 @@
 #include <libswscale/swscale.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_string (path, _("File"), "/tmp/fnord.mp4", _("Target path and filename, use '-' for stdout."))
-gegl_chant_double (bitrate, _("Bitrate"), 0.0, 100000000.0, 800000.0, _("target bitrate"))
-gegl_chant_double (fps, _("FPS"), 0.0, 100.0, 25, _("frames per second"))
+property_string (path, _("File"), "/tmp/fnord.mp4")
+    description (_("Target path and filename, use '-' for stdout."))
+
+property_double (bitrate, _("Target bitrate"), 800000.0)
+     value_range (0.0, 100000000.0)
+
+property_double (fps, _("Frames/second"), 25)
+     value_range (0.0, 100.0)
 
 #else
 
-#define GEGL_CHANT_TYPE_SINK
-#define GEGL_CHANT_C_FILE       "ff-save.c"
+#define GEGL_OP_SINK
+#define GEGL_OP_C_SOURCE ff-save.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
 #ifdef HAVE_LIBAVFORMAT_AVFORMAT_H
 #include <libavformat/avformat.h>
@@ -110,15 +115,15 @@ typedef struct
 #define DISABLE_AUDIO
 
 static void
-init (GeglChantO *o)
+init (GeglProperties *o)
 {
   static gint inited = 0; /*< this is actually meant to be static, only to be done once */
-  Priv       *p = (Priv*)o->chant_data;
+  Priv       *p = (Priv*)o->user_data;
 
   if (p == NULL)
     {
       p = g_new0 (Priv, 1);
-      o->chant_data = (void*) p;
+      o->user_data = (void*) p;
     }
 
   if (!inited)
@@ -166,11 +171,11 @@ static void close_video       (Priv            *p,
 void        close_audio       (Priv            *p,
                                AVFormatContext *oc,
                                AVStream        *st);
-static int  tfile             (GeglChantO      *self);
-static void write_video_frame (GeglChantO      *self,
+static int  tfile             (GeglProperties      *self);
+static void write_video_frame (GeglProperties      *self,
                                AVFormatContext *oc,
                                AVStream        *st);
-static void write_audio_frame (GeglChantO      *self,
+static void write_audio_frame (GeglProperties      *self,
                                AVFormatContext *oc,
                                AVStream        *st);
 
@@ -218,7 +223,7 @@ buffer_flush (Priv * p)
 }
 
 static void
-buffer_open (GeglChantOperation *op, int size)
+buffer_open (GeglOpOperation *op, int size)
 {
   Priv     *p = (Priv*)op->priv;
 
@@ -233,7 +238,7 @@ buffer_open (GeglChantOperation *op, int size)
 }
 
 static void
-buffer_close (GeglChantOperation *op)
+buffer_close (GeglOpOperation *op)
 {
   Priv     *p = (Priv*)op->priv;
 
@@ -351,7 +356,7 @@ buffer_write (Priv * p, uint8_t * source, int count)
 #ifndef DISABLE_AUDIO
 /* add an audio output stream */
 static AVStream *
-add_audio_stream (GeglChantOperation *op, AVFormatContext * oc, int codec_id)
+add_audio_stream (GeglOpOperation *op, AVFormatContext * oc, int codec_id)
 {
   Priv     *p = (Priv*)op->priv;
   AVCodecContext *c;
@@ -393,7 +398,7 @@ open_audio (Priv * p, AVFormatContext * oc, AVStream * st)
     }
 
   /* open it */
-  if (avcodec_open (c, codec) < 0)
+  if (avcodec_open2 (c, codec, NULL) < 0)
     {
       fprintf (stderr, "could not open codec\n");
       exit (1);
@@ -429,9 +434,9 @@ open_audio (Priv * p, AVFormatContext * oc, AVStream * st)
 }
 
 void
-write_audio_frame (GeglChantO *op, AVFormatContext * oc, AVStream * st)
+write_audio_frame (GeglProperties *op, AVFormatContext * oc, AVStream * st)
 {
-  Priv *p = (Priv*)op->chant_data;
+  Priv *p = (Priv*)op->user_data;
 
   AVCodecContext *c;
   AVPacket  pkt;
@@ -455,7 +460,7 @@ write_audio_frame (GeglChantO *op, AVFormatContext * oc, AVStream * st)
                                        p->audio_outbuf_size, p->samples);
 
       pkt.pts = c->coded_frame->pts;
-      pkt.flags |= PKT_FLAG_KEY;
+      pkt.flags |= AV_PKT_FLAG_KEY;
       pkt.stream_index = st->index;
       pkt.data = p->audio_outbuf;
 
@@ -480,9 +485,9 @@ close_audio (Priv * p, AVFormatContext * oc, AVStream * st)
 
 /* add a video output stream */
 static AVStream *
-add_video_stream (GeglChantO *op, AVFormatContext * oc, int codec_id)
+add_video_stream (GeglProperties *op, AVFormatContext * oc, int codec_id)
 {
-  Priv *p = (Priv*)op->chant_data;
+  Priv *p = (Priv*)op->user_data;
 
   AVCodecContext *c;
   AVStream *st;
@@ -496,7 +501,7 @@ add_video_stream (GeglChantO *op, AVFormatContext * oc, int codec_id)
 
   c = st->codec;
   c->codec_id = codec_id;
-  c->codec_type = CODEC_TYPE_VIDEO;
+  c->codec_type = AVMEDIA_TYPE_VIDEO;
 
   /* put sample propeters */
   c->bit_rate = op->bitrate;
@@ -624,24 +629,24 @@ close_video (Priv * p, AVFormatContext * oc, AVStream * st)
 
 /* prepare a dummy image */
 static void
-fill_yuv_image (GeglChantO *op,
+fill_yuv_image (GeglProperties *op,
                 AVFrame *pict, int frame_index, int width, int height)
 {
-  Priv     *p = (Priv*)op->chant_data;
+  Priv     *p = (Priv*)op->user_data;
   /*memcpy (pict->data[0],
 
    op->input_pad[0]->data,
 
           op->input_pad[0]->width * op->input_pad[0]->height * 3);*/
   GeglRectangle rect={0,0,width,height};
-  gegl_buffer_get (p->input, 1.0, &rect, babl_format ("R'G'B' u8"), pict->data[0], GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_get (p->input, &rect, 1.0, babl_format ("R'G'B' u8"), pict->data[0], GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 }
 
 static void
-write_video_frame (GeglChantO *op,
+write_video_frame (GeglProperties *op,
                    AVFormatContext *oc, AVStream *st)
 {
-  Priv     *p = (Priv*)op->chant_data;
+  Priv     *p = (Priv*)op->user_data;
   int       out_size, ret;
   AVCodecContext *c;
   AVFrame  *picture_ptr;
@@ -689,7 +694,7 @@ write_video_frame (GeglChantO *op,
       AVPacket  pkt;
       av_init_packet (&pkt);
 
-      pkt.flags |= PKT_FLAG_KEY;
+      pkt.flags |= AV_PKT_FLAG_KEY;
       pkt.stream_index = st->index;
       pkt.data = (uint8_t *) picture_ptr;
       pkt.size = sizeof (AVPicture);
@@ -712,7 +717,7 @@ write_video_frame (GeglChantO *op,
 
           pkt.pts = c->coded_frame->pts;
           if (c->coded_frame->key_frame)
-            pkt.flags |= PKT_FLAG_KEY;
+            pkt.flags |= AV_PKT_FLAG_KEY;
           pkt.stream_index = st->index;
           pkt.data = p->video_outbuf;
           pkt.size = out_size;
@@ -734,19 +739,19 @@ write_video_frame (GeglChantO *op,
 }
 
 static int
-tfile (GeglChantO *self)
+tfile (GeglProperties *self)
 {
-  Priv *p = (Priv*)self->chant_data;
+  Priv *p = (Priv*)self->user_data;
 
-  p->fmt = guess_format (NULL, self->path, NULL);
+  p->fmt = av_guess_format (NULL, self->path, NULL);
   if (!p->fmt)
     {
       fprintf (stderr,
                "ff_save couldn't deduce outputformat from file extension: using MPEG.\n%s",
                "");
-      p->fmt = guess_format ("mpeg", NULL, NULL);
+      p->fmt = av_guess_format ("mpeg", NULL, NULL);
     }
-  p->oc = av_alloc_format_context ();/*g_malloc (sizeof (AVFormatContext));*/
+  p->oc = avformat_alloc_context ();/*g_malloc (sizeof (AVFormatContext));*/
   if (!p->oc)
     {
       fprintf (stderr, "memory error\n%s", "");
@@ -795,7 +800,7 @@ tfile (GeglChantO *self)
 
 #if 0
 static int
-filechanged (GeglChantOperation *op, const char *att)
+filechanged (GeglOpOperation *op, const char *att)
 {
   init (op);
   return 0;
@@ -805,17 +810,18 @@ filechanged (GeglChantOperation *op, const char *att)
 static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *input,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
   static gint inited = 0;
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
-  Priv       *p = (Priv*)o->chant_data;
+  GeglProperties *o = GEGL_PROPERTIES (operation);
+  Priv       *p = (Priv*)o->user_data;
 
   g_assert (input);
 
   if (p == NULL)
     init (o);
-  p = (Priv*)o->chant_data;
+  p = (Priv*)o->user_data;
 
   p->width = result->width;
   p->height = result->height;
@@ -837,10 +843,10 @@ process (GeglOperation       *operation,
 static void
 finalize (GObject *object)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (object);
-  if (o->chant_data)
+  GeglProperties *o = GEGL_PROPERTIES (object);
+  if (o->user_data)
     {
-      Priv *p = (Priv*)o->chant_data;
+      Priv *p = (Priv*)o->user_data;
 
     if (p->oc)
       {
@@ -860,8 +866,8 @@ finalize (GObject *object)
         url_fclose (&p->oc->pb);
         free (p->oc);
       }
-      g_free (o->chant_data);
-      o->chant_data = NULL;
+      g_free (o->user_data);
+      o->user_data = NULL;
     }
 
   G_OBJECT_CLASS (g_type_class_peek_parent (G_OBJECT_GET_CLASS (object)))->finalize (object);
@@ -869,7 +875,7 @@ finalize (GObject *object)
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass     *operation_class;
   GeglOperationSinkClass *sink_class;
@@ -882,9 +888,11 @@ gegl_chant_class_init (GeglChantClass *klass)
   sink_class->process = process;
   sink_class->needs_full = TRUE;
 
-  operation_class->name        = "gegl:ff-save";
-  operation_class->categories  = "output:video";
-  operation_class->description = _("FFmpeg video output sink");
+  gegl_operation_class_set_keys (operation_class,
+    "name"        , "gegl:ff-save",
+    "categories"  , "output:video",
+    "description" , _("FFmpeg video output sink"),
+    NULL);
 }
 
 #endif
