@@ -20,16 +20,16 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
     /* No properties */
 
 #else
 
-#define GEGL_CHANT_TYPE_COMPOSER
-#define GEGL_CHANT_C_FILE       "hstack.c"
+#define GEGL_OP_COMPOSER
+#define GEGL_OP_C_SOURCE hstack.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <math.h>
 
 static void prepare (GeglOperation *operation)
@@ -106,14 +106,12 @@ process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *aux,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
-  GeglOperationComposer *composer;
   GeglBuffer            *temp_in;
   GeglBuffer            *temp_aux;
 
-  composer = GEGL_OPERATION_COMPOSER (operation);
-
   /* FIXME: just pass the originals buffers if the result rectangle does not
    * include both input buffers
    */
@@ -125,8 +123,10 @@ process (GeglOperation       *operation,
       gfloat *buf  = g_new0 (gfloat, result->width * result->height * 4);
       gfloat *bufB = g_new0 (gfloat, result->width * result->height * 4);
 
-      gegl_buffer_get (temp_in, 1.0, NULL, babl_format ("RGBA float"), buf, GEGL_AUTO_ROWSTRIDE);
-      gegl_buffer_get (temp_aux, 1.0, NULL, babl_format ("RGBA float"), bufB, GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_get (temp_in,  NULL, 1.0, babl_format ("RGBA float"), buf,
+                       GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
+      gegl_buffer_get (temp_aux, NULL, 1.0, babl_format ("RGBA float"), bufB,
+                       GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
         {
           gint offset=0;
           gint x,y;
@@ -143,7 +143,7 @@ process (GeglOperation       *operation,
                 offset+=4;
               }
         }
-      gegl_buffer_set (output, NULL, babl_format ("RGBA float"), buf,
+      gegl_buffer_set (output, NULL, 0, babl_format ("RGBA float"), buf,
                        GEGL_AUTO_ROWSTRIDE);
 
       g_free (buf);
@@ -157,7 +157,7 @@ process (GeglOperation       *operation,
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass         *operation_class;
   GeglOperationComposerClass *composer_class;
@@ -171,10 +171,12 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_invalidated_by_change = get_invalidated_by_change;
   operation_class->get_required_for_output = get_required_for_output;
 
-  operation_class->name        = "gegl:hstack";
-  operation_class->categories  = "misc";
-  operation_class->description =
-        _("Horizontally stack inputs, (in \"output\" \"aux\" is placed to the right of \"input\")");
+  gegl_operation_class_set_keys (operation_class,
+  "name"        , "gegl:hstack",
+  "categories"  , "misc",
+  "description" ,
+        _("Horizontally stack inputs, (in \"output\" \"aux\" is placed to the right of \"input\")"),
+        NULL);
 }
 
 #endif
